import InsertDrupalLayoutCommand from "./insertdrupallayoutcommand";
import { Plugin } from 'ckeditor5/src/core';
import { Widget, toWidget, toWidgetEditable } from 'ckeditor5/src/widget';

export default class InsertDrupalLayoutEditing extends Plugin {
  static get requires() {
    return [ Widget ];
  }

  init() {
    this._defineSchema();
    this._defineConverters();

    this.editor.commands.add('insertDrupalLayout', new InsertDrupalLayoutCommand(this.editor));
  }

  _defineSchema() {
    const editor = this.editor;
    const schema = editor.model.schema;

    schema.register('drupalLayout', {
      inheritAllFrom: '$blockObject',
    });

    schema.register('drupalLayoutRow', {
      allowIn: 'drupalLayout',
      allowContentOf: 'drupalLayoutColumn',
      allowAttributes: [ 'tagName', 'tagAttributes' ],
    });

    schema.register('drupalLayoutColumn', {
      allowIn: 'drupalLayoutRow',
      allowContentOf: '$root',
      allowAttributes: [ 'tagName', 'tagAttributes' ],
    });
  }

  _defineConverters() {
    const editor = this.editor;
    const conversion = editor.conversion;

    // <drupalLayout>
    conversion.for('upcast').elementToElement({
      model: 'drupalLayout',
      view: {
        name: 'div',
        classes: ['drupal-layout']
      }
    });
    // Convert to DOM for rendering on page.
    conversion.for('dataDowncast').elementToElement({
      model: 'drupalLayout',
      view: {
        name: 'div',
        classes: ['drupal-layout']
      }
    });
    // Convert for editing in the editor.
    conversion.for('editingDowncast').elementToElement({
      model: 'drupalLayout',
      view: (modelElement, { writer }) => {
        const drupalLayout = writer.createContainerElement('div', { class: 'drupal-layout' });

        return toWidget(drupalLayout, writer, { label: 'Drupal layout' });
      }
    });

    // <drupalLayoutRow>
    conversion.for('upcast').elementToElement({
      model: (viewElement, { writer }) => {
        const rowAttributes = [];
        viewElement.getAttributeKeys().forEach((key) => {
          rowAttributes.push({ name: key, value: viewElement.getAttribute(key) });
        });
        return writer.createElement('drupalLayoutRow', { tagName: viewElement.name, tagAttributes: JSON.stringify(rowAttributes) });
      },
      view: {
        classes: ['drupal-layout__row']
      }
    });
    // Convert to DOM for rendering on page.
    conversion.for('dataDowncast').elementToElement({
      model: 'drupalLayoutRow',
      view: (modelElement, { writer }) => {
        const attributesJson = modelElement.getAttribute('tagAttributes') ?? '{}';
        const rowAttributes = {};
        JSON.parse(attributesJson).forEach((attribute) => {
          rowAttributes[attribute.name] = attribute.value;
        });
        const drupalLayoutRow = writer.createContainerElement(modelElement.getAttribute('tagName'), rowAttributes);

        // Add a class, so we can upcast the element later.
        let rowClasses = drupalLayoutRow.getAttribute('class');
        rowClasses = rowClasses === undefined ? 'drupal-layout__row' : rowClasses + ' drupal-layout__row';
        writer.setAttribute('class', rowClasses.trim(), drupalLayoutRow);

        return drupalLayoutRow;
      }
    });
    // Convert for editing in the editor.
    conversion.for('editingDowncast').elementToElement({
      model: 'drupalLayoutRow',
      view: (modelElement, { writer }) => {
        const attributesJson = modelElement.getAttribute('tagAttributes') ?? '{}';
        const rowAttributes = {};
        JSON.parse(attributesJson).forEach((attribute) => {
          rowAttributes[attribute.name] = attribute.value;
        });
        const drupalLayoutRow = writer.createContainerElement(modelElement.getAttribute('tagName'), rowAttributes);

        // Add a class, so we can add a border around the column.
        let rowClasses = drupalLayoutRow.getAttribute('class');
        rowClasses = rowClasses === undefined ? 'drupal-layout__row' : rowClasses + ' drupal-layout__row';
        writer.setAttribute('class', rowClasses.trim(), drupalLayoutRow);

        return toWidget(drupalLayoutRow, writer, { label: 'Drupal layout row' });
      }
    });

    // <drupalLayoutColumn>
    conversion.for('upcast').elementToElement({
      model: (viewElement, { writer }) => {
        const columnAttributes = [];
        viewElement.getAttributeKeys().forEach((key) => {
          columnAttributes.push({ name: key, value: viewElement.getAttribute(key) });
        });

        return writer.createElement('drupalLayoutColumn', { tagName: viewElement.name, tagAttributes: JSON.stringify(columnAttributes) });
      },
      view: {
        classes: ['drupal-layout__column']
      }
    });
    // Convert to DOM for rendering on page.
    conversion.for('dataDowncast').elementToElement({
      model: 'drupalLayoutColumn',
      view: (modelElement, { writer }) => {
        const attributesJson = modelElement.getAttribute('tagAttributes') ?? '{}';
        const columnAttributes = {};
        JSON.parse(attributesJson).forEach((attribute) => {
          columnAttributes[attribute.name] = attribute.value;
        });
        const drupalLayoutColumn = writer.createContainerElement(modelElement.getAttribute('tagName'), columnAttributes);

        // Add a class, so we can upcast the element later.
        let columnClasses = drupalLayoutColumn.getAttribute('class');
        columnClasses = columnClasses === undefined ? 'drupal-layout__column' : columnClasses + ' drupal-layout__column';
        writer.setAttribute('class', columnClasses.trim(), drupalLayoutColumn);

        return drupalLayoutColumn;
      }
    });
    // Convert for editing in the editor.
    conversion.for('editingDowncast').elementToElement({
      model: 'drupalLayoutColumn',
      view: (modelElement, { writer }) => {
        const attributesJson = modelElement.getAttribute('tagAttributes') ?? '{}';
        const columnAttributes = {};
        JSON.parse(attributesJson).forEach((attribute) => {
          columnAttributes[attribute.name] = attribute.value;
        });
        const drupalLayoutColumn = writer.createContainerElement(modelElement.getAttribute('tagName'), columnAttributes);

        // Add a class, so we can add a border around the column.
        let columnClasses = drupalLayoutColumn.getAttribute('class');
        columnClasses = columnClasses === undefined ? 'drupal-layout__column' : columnClasses + ' drupal-layout__column';
        writer.setAttribute('class', columnClasses.trim(), drupalLayoutColumn);

        return toWidgetEditable(drupalLayoutColumn, writer, { label: 'Drupal layout column' });
      }
    });
  }
}
