<?php

namespace Drupal\ckeditor_layouts\Plugin\CKEditor5Plugin;

use Drupal\ckeditor5\HTMLRestrictions;
use Drupal\ckeditor5\Plugin\CKEditor5PluginConfigurableInterface;
use Drupal\ckeditor5\Plugin\CKEditor5PluginConfigurableTrait;
use Drupal\ckeditor5\Plugin\CKEditor5PluginDefault;
use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\editor\EditorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a CKEditor 5 plugin to insert layouts.
 *
 * @CKEditor5Plugin(
 *   id = "ckeditor_layouts_drupal_layouts",
 *   ckeditor5 = @CKEditor5AspectsOfCKEditor5Plugin(
 *     plugins = {
 *       "drupalLayouts.InsertDrupalLayout",
 *     }
 *   ),
 *   drupal = @DrupalAspectsOfCKEditor5Plugin(
 *     label = @Translation("Layouts"),
 *     library = "ckeditor_layouts/ckeditor5.layouts",
 *     admin_library = "ckeditor_layouts/admin.ckeditor5.layouts",
 *     toolbar_items = {
 *       "insertDrupalLayout" = {
 *         "label" = @Translation("Insert layout"),
 *       },
 *     },
 *     elements = { "<div>", "<div class>" },
 *   )
 * )
 */
class Layouts extends CKEditor5PluginDefault implements ContainerFactoryPluginInterface, CKEditor5PluginConfigurableInterface {

  use CKEditor5PluginConfigurableTrait;

  /**
   * The layout plugin manager.
   *
   * @var \Drupal\Core\Layout\LayoutPluginManagerInterface
   */
  protected $layoutPluginManager;

  /**
   * The library discovery service.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface
   */
  protected $libraryDiscovery;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The module list service.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleList;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $instance = new static($configuration, $plugin_id, $plugin_definition);

    $instance->layoutPluginManager = $container->get('plugin.manager.core.layout');
    $instance->libraryDiscovery = $container->get('library.discovery');
    $instance->renderer = $container->get('renderer');
    $instance->moduleList = $container->get('extension.list.module');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return ['enabled_layouts' => []];
  }

  /**
   * Computes all valid choices for the "enabled_layouts" setting.
   *
   * @see ckeditor_layouts.schema.yml
   *
   * @return string[]
   *   All valid choices.
   */
  public static function validChoices(): array {
    $valid_choices = [];

    $layout_plugins = \Drupal::service('plugin.manager.core.layout')->getSortedDefinitions();
    foreach ($layout_plugins as $plugin) {
      $valid_choices[] = $plugin->id();
    }

    return $valid_choices;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    // @TODO: We should check the structure of layouts to make sure our
    //   CKEditor 5 plugin can handle them.
    $form['enabled_layouts'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Enabled layouts'),
      '#description' => $this->t('Only the layouts that are selected here are available in the editor.'),
    ];

    $layout_plugins = $this->layoutPluginManager->getSortedDefinitions();
    foreach ($layout_plugins as $plugin) {
      // The layout_builder_blank plugin is for internal use by Layout Builder
      // only.
      if ($plugin->id() === 'layout_builder_blank') {
        continue;
      }

      $form['enabled_layouts'][$plugin->id()] = [
        '#type' => 'checkbox',
        '#title' => $plugin->getLabel(),
        '#return_value' => $plugin->id(),
        '#default_value' => in_array($plugin->id(), $this->configuration['enabled_layouts'], TRUE) ? $plugin->id() : NULL,
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
    // Match the config schema structure at
    // ckeditor5.plugin.ckeditor_layouts_drupal_layout.
    $form_value = $form_state->getValue('enabled_layouts');
    $config_value = array_values(array_filter($form_value));
    $form_state->setValue('enabled_layouts', $config_value);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['enabled_layouts'] = $form_state->getValue('enabled_layouts');
    // @todo Do we need to clear the plugin cache?
  }

  /**
   * {@inheritdoc}
   */
  public function getDynamicPluginConfig(array $static_plugin_config, EditorInterface $editor): array {
    $dynamic_plugin_config = [
      'drupalLayouts' => [
        'layouts' => [],
      ],
    ];

    // Get the plugins for enabled layouts.
    $enabled_layouts = $editor->getSettings()['plugins']['ckeditor_layouts_drupal_layouts']['enabled_layouts'] ?? [];
    $layouts = array_filter($this->layoutPluginManager->getSortedDefinitions(), function ($layout) use ($enabled_layouts) {
      if (in_array($layout->id(), $enabled_layouts)) {
        return TRUE;
      }

      return FALSE;
    });

    $allowed_tags = [];
    foreach ($layouts as $layout) {
      // Prepare the icon for rendering.
      $icon = $layout->getIcon(24, 24, 2, 2);

      // Prepare the template for rendering. Rendering is based on a theme hook.
      // Rendering of regions in a layout might be conditional based on actual
      // content in the region. In the editor, we always want all regions to be
      // rendered so that content can be added in any available region.
      // Therefore, we put an empty markup element in each region.
      $template = ['#theme' => $layout->getThemeHook()];
      foreach ($layout->getRegionNames() as $region) {
        $template[$region] = ['#markup' => ''];
      }

      // Add the layout to the JavaScript settings.
      $layout_html = preg_replace('/<!--(.|\s)*?-->/', '', $this->renderer->render($template));
      $dynamic_plugin_config['drupalLayouts']['layouts'][] = [
        'id' => $layout->id(),
        'label' => $layout->getLabel(),
        // Add the rendered icon and template. We filter out HTML comments
        // just in case Twig debug is enabled.
        'image' => preg_replace('/<!--(.|\s)*?-->/', '', $this->renderer->render($icon)),
        'template' => $layout_html,
      ];

      // Extract all tags from the layout's template.
      $xml = Html::load($layout_html);
      $childs_to_process[] = $xml->childNodes;
      while (!empty($childs_to_process)) {
        $childs = array_pop($childs_to_process);
        foreach ($childs as $child) {
          if (!str_starts_with($child->nodeName, '#') && !in_array($child->nodeName, ['html', 'head', 'meta', 'body'])) {
            $attribute_list = [];
            if ($child->hasAttributes()) {
              foreach ($child->attributes as $attribute) {
                $attribute_list[] = $attribute->name;
              }
            }
            $attributes = implode(' ', $attribute_list);
            $allowed_tags[] = "<$child->nodeName $attributes>";
          }
          if ($child->hasChildNodes()) {
            $childs_to_process[] = $child->childNodes;
          }
        }
      }
    }

    // Pass the allowed tags to CKEditor.
    $restrictions = HTMLRestrictions::fromString(implode(' ', array_unique($allowed_tags)));
    $dynamic_plugin_config['htmlSupport'] = [
      'allow' => $restrictions->toGeneralHtmlSupportConfig(),
      // Any inserted elements are explicitly allowed to be empty.
      'allowEmpty' => array_keys($restrictions->getAllowedElements()),
    ];

    return array_merge($static_plugin_config, $dynamic_plugin_config);
  }

}
